#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <windows.h>
#include <commctrl.h>
#include <winioctl.h>
#include "resource.h"

extern int g_NSECTORS;
extern int g_DSECTORS;
extern int g_BUFSIZE;

#define SC_HA_INQUIRY    0
#define SC_GET_DEV_TYPE  1
#define SC_EXEC_SCSI_CMD 2
#define SC_ABORT_SRB     3
#define SC_RESET_DEV     4
#define SC_GET_DISK_INFO 5

#define SENSE_LEN 14
 
#define SRB_DIR_IN       0x08
#define SRB_DIR_OUT      0x10
#define SRB_EVENT_NOTIFY 0x40

#define TIMEOUT 5000
#define MAX_CDROM 10

#define SYNC_SIZE 320

#define OVERLAP 3
#define NSECTORS g_NSECTORS
#define DSECTORS g_DSECTORS
#define SECTOR_SIZE 2352L
#define BUFSIZE g_BUFSIZE
#define TOTAL_SIZE (NSECTORS*SECTOR_SIZE)
#define OVERLAP_SIZE ((NSECTORS+OVERLAP)*SECTOR_SIZE)
#define SYNCHRO_SIZE (OVERLAP*SECTOR_SIZE)
#define DTOTAL_SIZE (DSECTORS*SECTOR_SIZE)
#define DOVERLAP_SIZE ((DSECTORS+OVERLAP)*SECTOR_SIZE)

typedef enum { RM_GENERAL, RM_PLEXTOR, RM_NEC, RM_TOSHIBA, RM_ATAPI } ReadMethod;
typedef enum { SL_PLEXTOR, SL_HP4020, SL_HP6020, SL_GENERIC, SL_ATAPI, SL_NONE } SpeedSelect;
typedef enum { MS_GENERAL, MS_TOSHIBA, MS_NONE, MS_ATAPI } LModeSelect;

typedef struct {
  BYTE  SRB_Cmd;
  BYTE  SRB_Status;
  BYTE  SRB_HaId;
  BYTE  SRB_Flags;
  DWORD SRB_Hdr_Rsvd;
  BYTE  HA_Count;
  BYTE  HA_SCSI_ID;
  BYTE  HA_ManagerId[16];
  BYTE  HA_Identifier[16];
  BYTE  HA_Unique[16];
  WORD  HA_Rsvd1;
} SRB_HAInquiry;

typedef struct {
  BYTE  SRB_Cmd;
  BYTE  SRB_Status;
  BYTE  SRB_HaId;
  BYTE  SRB_Flags;
  DWORD SRB_Hdr_Rsvd;
  BYTE  SRB_Target;
  BYTE  SRB_Lun;
  WORD  SRB_Rsvd1;
  DWORD SRB_BufLen;
  BYTE  *SRB_BufPointer;
  BYTE  SRB_SenseLen;
  BYTE  SRB_CDBLen;
  BYTE  SRB_HaStat;
  BYTE  SRB_TargStat;
  void  (*SRB_PostProc)();
  void  *SRB_Rsvd2;
  BYTE  SRB_Rsvd3[16];
  BYTE  SRB_CDBByte[16];
  BYTE  SenseArea[SENSE_LEN+2];
} SRB_ExecSCSICmd;

typedef struct {
  int         lun;
  int         scsi_id;
  int         ha_id;
  ReadMethod  readmethod;
  LModeSelect modeselect;
  SpeedSelect speedselect;
  int         nspeeds;
  int         speed[10];
  int         bigendian; // returned bytes are reversed
  int         jitter;    // drive requires jitter correction
  char        desc[50];
} CDROM;

typedef struct {
  BYTE reserved;
  BYTE adr_control;
  BYTE track;
  BYTE reserved2;
  BYTE address1;
  BYTE address2;
  BYTE address3;
  BYTE address4;
} TrackDef;

typedef struct {
  WORD data_length;
  BYTE first_track;
  BYTE last_track;
  TrackDef tracks[100];
} TOC;

struct FORMAT {
	char fID[4];
	DWORD fLen;
	WORD wTag;
	WORD wChannel;
	DWORD nSample;
	DWORD nByte;
	WORD align;
	WORD sample;
	};

struct RIFF {
        char rID[4];
	DWORD rLen;
};

struct DATA {
	char dID[4];
	DWORD dLen;
};

struct WAVE {
	char wID[4];
	struct FORMAT fmt;
	struct DATA data;
};

typedef struct {
	struct RIFF riff;
	struct WAVE wave;
} WaveFile;

void Error(HWND hWnd,char *str);
void DisplayLastError(HWND hWnd);
int InitializeASPI(void);
void GetCDRomDrives(HWND hWnd);
void DisposeASPI(void);
void ReadTOC(HWND hWnd,int cd,int compare);
void Working(LPTHREAD_START_ROUTINE routine,HWND hWnd);
DWORD WINAPI GrabTrack(LPVOID hWnd);
void ShowSpeeds(HWND hWnd,int cd);
void DisplayProgress(HWND hWnd);
void PlayAudio(int cd,int track);
void StopAudio(void);
void NextTrack(void);
void PreviousTrack(void);
void CalculateSpeeds(int cd,int speed);
void GetTargetFile(HWND hWnd,char *filename);
void ReadCDSettings(int cd,char *desc);
void WriteCDSettings(int cd,char *desc);
int SetSpeed(int cd,int speed);

extern HINSTANCE hInst;
extern HWND g_hWnd,whWnd;
extern int cd,track,stopthread1,do_compare,use_tmpfile,max_speed,skipverify,skiperror,autoskip,skipfile,g_directdisk,n_cdrom,showspeed;
extern char filename[255],h_filename[255];
extern TOC toc;
extern char drive[4];
extern CDROM cdrom[MAX_CDROM];
FARPROC GetASPI32SupportInfo;
FARPROC SendASPI32Command;
